@@ -17,8 +17,8 @@
 package org.apache.commons.pool2.impl;
 
 import java.time.Duration;
+import java.time.Instant;
 import java.util.ArrayList;
-import java.util.Map;
 import java.util.NoSuchElementException;
 import java.util.Set;
 import java.util.concurrent.ConcurrentHashMap;
@@ -86,6 +86,10 @@ public class GenericObjectPool<T> extends BaseGenericObjectPool<T>
     private static final String ONAME_BASE =
         "org.apache.commons.pool2:type=GenericObjectPool,name=";
 
+    private static void wait(final Object obj, final Duration duration) throws InterruptedException {
+        obj.wait(duration.toMillis(), duration.getNano() % 1_000_000);
+    }
+
     private volatile String factoryType;
 
     private volatile int maxIdle = GenericObjectPoolConfig.DEFAULT_MAX_IDLE;
@@ -101,8 +105,7 @@ public class GenericObjectPool<T> extends BaseGenericObjectPool<T>
      * #_maxActive}. Map keys are pooled objects, values are the PooledObject
      * wrappers used internally by the pool.
      */
-    private final Map<IdentityWrapper<T>, PooledObject<T>> allObjects =
-        new ConcurrentHashMap<>();
+    private final ConcurrentHashMap<IdentityWrapper<T>, PooledObject<T>> allObjects = new ConcurrentHashMap<>();
 
     /*
      * The combined count of the currently created objects and those in the
@@ -186,7 +189,7 @@ public class GenericObjectPool<T> extends BaseGenericObjectPool<T>
      * @throws Exception If the factory fails to passivate the object
      */
     private void addIdleObject(final PooledObject<T> p) throws Exception {
-        if (p != null) {
+        if (!PooledObject.isNull(p)) {
             factory.passivateObject(p);
             if (getLifo()) {
                 idleObjects.addFirst(p);
@@ -201,7 +204,14 @@ public class GenericObjectPool<T> extends BaseGenericObjectPool<T>
      * "pre-loading" a pool with idle objects.
      * <p>
      * If there is no capacity available to add to the pool, this is a no-op
-     * (no exception, no impact to the pool). </p>
+     * (no exception, no impact to the pool).
+     * </p>
+     * <p>
+     * If the factory returns null when creating an object, a {@code NullPointerException}
+     * is thrown. If there is no factory set (factory == null), an {@code IllegalStateException}
+     * is thrown.
+     * </p>
+     * 
      */
     @Override
     public void addObject() throws Exception {
@@ -243,7 +253,8 @@ public class GenericObjectPool<T> extends BaseGenericObjectPool<T>
      * checked out from the pool is less than {@code maxTotal,} a new
      * instance is created, activated and (if applicable) validated and returned
      * to the caller. If validation fails, a {@code NoSuchElementException}
-     * is thrown.
+     * is thrown. If the factory returns null when creating an instance,
+     * a {@code NullPointerException} is thrown.
      * </p>
      * <p>
      * If the pool is exhausted (no available idle instances and no capacity to
@@ -266,19 +277,16 @@ public class GenericObjectPool<T> extends BaseGenericObjectPool<T>
      *                            to become available
      *
      * @return object instance from the pool
-     *
      * @throws NoSuchElementException if an instance cannot be returned
-     *
-     * @throws Exception if an object instance cannot be returned due to an
-     *                   error
+     * @throws Exception if an object instance cannot be returned due to an error
      * @since 2.10.0
      */
     public T borrowObject(final Duration borrowMaxWaitDuration) throws Exception {
         assertOpen();
 
         final AbandonedConfig ac = this.abandonedConfig;
-        if (ac != null && ac.getRemoveAbandonedOnBorrow() && (getNumIdle() < 2) &&
-                (getNumActive() > getMaxTotal() - 3)) {
+        if (ac != null && ac.getRemoveAbandonedOnBorrow() && getNumIdle() < 2 &&
+                getNumActive() > getMaxTotal() - 3) {
             removeAbandoned(ac);
         }
 
@@ -289,44 +297,40 @@ public class GenericObjectPool<T> extends BaseGenericObjectPool<T>
         final boolean blockWhenExhausted = getBlockWhenExhausted();
 
         boolean create;
-        final long waitTimeMillis = System.currentTimeMillis();
+        final Instant waitTime = Instant.now();
 
         while (p == null) {
             create = false;
             p = idleObjects.pollFirst();
             if (p == null) {
                 p = create();
-                if (p != null) {
+                if (!PooledObject.isNull(p)) {
                     create = true;
                 }
             }
             if (blockWhenExhausted) {
-                if (p == null) {
-                    if (borrowMaxWaitDuration.isNegative()) {
-                        p = idleObjects.takeFirst();
-                    } else {
-                        p = idleObjects.pollFirst(borrowMaxWaitDuration);
-                    }
+                if (PooledObject.isNull(p)) {
+                    p = borrowMaxWaitDuration.isNegative() ? idleObjects.takeFirst() : idleObjects.pollFirst(borrowMaxWaitDuration);
                 }
-                if (p == null) {
+                if (PooledObject.isNull(p)) {
                     throw new NoSuchElementException(appendStats(
                             "Timeout waiting for idle object, borrowMaxWaitDuration=" + borrowMaxWaitDuration));
                 }
-            } else if (p == null) {
+            } else if (PooledObject.isNull(p)) {
                 throw new NoSuchElementException(appendStats("Pool exhausted"));
             }
             if (!p.allocate()) {
                 p = null;
             }
 
-            if (p != null) {
+            if (!PooledObject.isNull(p)) {
                 try {
                     factory.activateObject(p);
                 } catch (final Exception e) {
                     try {
                         destroy(p, DestroyMode.NORMAL);
-                    } catch (final Exception e1) {
-                        // Ignore - activation failure is more important
+                    } catch (final Exception ignored) {
+                        // ignored - activation failure is more important
                     }
                     p = null;
                     if (create) {
@@ -336,7 +340,7 @@ public class GenericObjectPool<T> extends BaseGenericObjectPool<T>
                         throw nsee;
                     }
                 }
-                if (p != null && getTestOnBorrow()) {
+                if (!PooledObject.isNull(p) && getTestOnBorrow()) {
                     boolean validate = false;
                     Throwable validationThrowable = null;
                     try {
@@ -349,8 +353,8 @@ public class GenericObjectPool<T> extends BaseGenericObjectPool<T>
                         try {
                             destroy(p, DestroyMode.NORMAL);
                             destroyedByBorrowValidationCount.incrementAndGet();
-                        } catch (final Exception e) {
-                            // Ignore - validation failure is more important
+                        } catch (final Exception ignored) {
+                            // ignored - validation failure is more important
                         }
                         p = null;
                         if (create) {
@@ -364,23 +368,11 @@ public class GenericObjectPool<T> extends BaseGenericObjectPool<T>
             }
         }
 
-        updateStatsBorrow(p, Duration.ofMillis(System.currentTimeMillis() - waitTimeMillis));
+        updateStatsBorrow(p, Duration.between(waitTime, Instant.now()));
 
         return p.getObject();
     }
 
-    PooledObject<T> getPooledObject(final T obj) {
-        return allObjects.get(new IdentityWrapper<>(obj));
-    }
-
-    @Override
-    String getStatsString() {
-        // Simply listed in AB order.
-        return super.getStatsString() +
-                String.format(", createdCount=%,d, makeObjectCount=%,d, maxIdle=%,d, minIdle=%,d",
-                        createdCount.get(), makeObjectCount, maxIdle, minIdle);
-    }
-
     /**
      * Borrows an object from the pool using the specific waiting time which only
      * applies if {@link #getBlockWhenExhausted()} is true.
@@ -401,7 +393,8 @@ public class GenericObjectPool<T> extends BaseGenericObjectPool<T>
      * checked out from the pool is less than {@code maxTotal,} a new
      * instance is created, activated and (if applicable) validated and returned
      * to the caller. If validation fails, a {@code NoSuchElementException}
-     * is thrown.
+     * is thrown. If the factory returns null when creating an instance,
+     * a {@code NullPointerException} is thrown.
      * </p>
      * <p>
      * If the pool is exhausted (no available idle instances and no capacity to
@@ -504,12 +497,14 @@ public class GenericObjectPool<T> extends BaseGenericObjectPool<T>
     /**
      * Attempts to create a new wrapped pooled object.
      * <p>
-     * If there are {@link #getMaxTotal()} objects already in circulation
-     * or in process of being created, this method returns null.
+     * If there are {@link #getMaxTotal()} objects already in circulation or in process of being created, this method
+     * returns null.
+     * </p>
+     * <p>
+     * If the factory makeObject returns null, this method throws a NullPointerException.
      * </p>
      *
-     * @return The new wrapped pooled object
-     *
+     * @return The new wrapped pooled object or null.
      * @throws Exception if the object factory's {@code makeObject} fails
      */
     private PooledObject<T> create() throws Exception {
@@ -519,8 +514,9 @@ public class GenericObjectPool<T> extends BaseGenericObjectPool<T>
             localMaxTotal = Integer.MAX_VALUE;
         }
 
-        final long localStartTimeMillis = System.currentTimeMillis();
-        final long localMaxWaitTimeMillis = Math.max(getMaxWaitDuration().toMillis(), 0);
+        final Instant localStartInstant = Instant.now();
+        final Duration maxWaitDurationRaw = getMaxWaitDuration();
+        final Duration localMaxWaitDuration = maxWaitDurationRaw.isNegative() ? Duration.ZERO : maxWaitDurationRaw;
 
         // Flag that indicates if create should:
         // - TRUE:  call the factory to create an object
@@ -545,7 +541,7 @@ public class GenericObjectPool<T> extends BaseGenericObjectPool<T>
                         // bring the pool to capacity. Those calls might also
                         // fail so wait until they complete and then re-test if
                         // the pool is at capacity or not.
-                        makeObjectCountLock.wait(localMaxWaitTimeMillis);
+                        wait(makeObjectCountLock, localMaxWaitDuration);
                     }
                 } else {
                     // The pool is not at capacity. Create a new object.
@@ -555,9 +551,8 @@ public class GenericObjectPool<T> extends BaseGenericObjectPool<T>
             }
 
             // Do not block more if maxWaitTimeMillis is set.
-            if (create == null &&
-                (localMaxWaitTimeMillis > 0 &&
-                 System.currentTimeMillis() - localStartTimeMillis >= localMaxWaitTimeMillis)) {
+            if (create == null && localMaxWaitDuration.compareTo(Duration.ZERO) > 0 &&
+                    Duration.between(localStartInstant, Instant.now()).compareTo(localMaxWaitDuration) >= 0) {
                 create = Boolean.FALSE;
             }
         }
@@ -569,6 +564,10 @@ public class GenericObjectPool<T> extends BaseGenericObjectPool<T>
         final PooledObject<T> p;
         try {
             p = factory.makeObject();
+            if (PooledObject.isNull(p)) {
+                createCount.decrementAndGet();
+                throw new NullPointerException(String.format("%s.makeObject() = null", factory.getClass().getSimpleName()));
+            }
             if (getTestOnCreate() && !factory.validateObject(p)) {
                 createCount.decrementAndGet();
                 return null;
@@ -623,19 +622,23 @@ public class GenericObjectPool<T> extends BaseGenericObjectPool<T>
      * {@link #getMaxTotal()}. If {@code always} is false, no instances are created unless
      * there are threads waiting to check out instances from the pool.
      * </p>
+     * <p>
+     * If the factory returns null when creating an instance, a {@code NullPointerException}
+     * is thrown.
+     * </p>
      *
      * @param idleCount the number of idle instances desired
      * @param always true means create instances even if the pool has no threads waiting
      * @throws Exception if the factory's makeObject throws
      */
     private void ensureIdle(final int idleCount, final boolean always) throws Exception {
-        if (idleCount < 1 || isClosed() || (!always && !idleObjects.hasTakeWaiters())) {
+        if (idleCount < 1 || isClosed() || !always && !idleObjects.hasTakeWaiters()) {
             return;
         }
 
         while (idleObjects.size() < idleCount) {
             final PooledObject<T> p = create();
-            if (p == null) {
+            if (PooledObject.isNull(p)) {
                 // Can't create objects, no reason to think another call to
                 // create will work. Give up.
                 break;
@@ -766,10 +769,9 @@ public class GenericObjectPool<T> extends BaseGenericObjectPool<T>
                                 }
                             }
                         }
-                        if (!underTest.endEvictionTest(idleObjects)) {
-                            // TODO - May need to add code here once additional
-                            // states are used
-                        }
+                        underTest.endEvictionTest(idleObjects);
+                        // TODO - May need to add code here once additional
+                        // states are used
                     }
                 }
             }
@@ -846,15 +848,12 @@ public class GenericObjectPool<T> extends BaseGenericObjectPool<T>
      *
      * @see #setMinIdle(int)
      * @see #setMaxIdle(int)
-     * @see #setTimeBetweenEvictionRuns(Duration)
+     * @see #setDurationBetweenEvictionRuns(Duration)
      */
     @Override
     public int getMinIdle() {
         final int maxIdleSave = getMaxIdle();
-        if (this.minIdle > maxIdleSave) {
-            return maxIdleSave;
-        }
-        return minIdle;
+        return Math.min(this.minIdle, maxIdleSave);
     }
 
     @Override
@@ -878,8 +877,8 @@ public class GenericObjectPool<T> extends BaseGenericObjectPool<T>
         if (numTestsPerEvictionRun >= 0) {
             return Math.min(numTestsPerEvictionRun, idleObjects.size());
         }
-        return (int) (Math.ceil(idleObjects.size() /
-                Math.abs((double) numTestsPerEvictionRun)));
+        return (int) Math.ceil(idleObjects.size() /
+                Math.abs((double) numTestsPerEvictionRun));
     }
 
     /**
@@ -898,15 +897,26 @@ public class GenericObjectPool<T> extends BaseGenericObjectPool<T>
         return 0;
     }
 
+    PooledObject<T> getPooledObject(final T obj) {
+        return allObjects.get(new IdentityWrapper<>(obj));
+    }
+
+    @Override
+    String getStatsString() {
+        // Simply listed in AB order.
+        return super.getStatsString() +
+                String.format(", createdCount=%,d, makeObjectCount=%,d, maxIdle=%,d, minIdle=%,d",
+                        createdCount.get(), makeObjectCount, maxIdle, minIdle);
+    }
+
     /**
      * {@inheritDoc}
      * <p>
-     * Activation of this method decrements the active count and attempts to
-     * destroy the instance, using the default (NORMAL) {@link DestroyMode}.
+     * Activation of this method decrements the active count and attempts to destroy the instance, using the default
+     * (NORMAL) {@link DestroyMode}.
      * </p>
      *
-     * @throws Exception             if an exception occurs destroying the
-     *                               object
+     * @throws Exception if an exception occurs destroying the
      * @throws IllegalStateException if obj does not belong to this pool
      */
     @Override
@@ -917,12 +927,11 @@ public class GenericObjectPool<T> extends BaseGenericObjectPool<T>
     /**
      * {@inheritDoc}
      * <p>
-     * Activation of this method decrements the active count and attempts to
-     * destroy the instance, using the provided {@link DestroyMode}.
+     * Activation of this method decrements the active count and attempts to destroy the instance, using the provided
+     * {@link DestroyMode}.
      * </p>
      *
-     * @throws Exception             if an exception occurs destroying the
-     *                               object
+     * @throws Exception if an exception occurs destroying the object
      * @throws IllegalStateException if obj does not belong to this pool
      * @since 2.9.0
      */
@@ -933,8 +942,7 @@ public class GenericObjectPool<T> extends BaseGenericObjectPool<T>
             if (isAbandonedConfig()) {
                 return;
             }
-            throw new IllegalStateException(
-                    "Invalidated object not currently part of this pool");
+            throw new IllegalStateException("Invalidated object not currently part of this pool");
         }
         synchronized (p) {
             if (p.getState() != PooledObjectState.INVALID) {
@@ -1177,7 +1185,10 @@ public class GenericObjectPool<T> extends BaseGenericObjectPool<T>
     public void use(final T pooledObject) {
         final AbandonedConfig abandonedCfg = this.abandonedConfig;
         if (abandonedCfg != null && abandonedCfg.getUseUsageTracking()) {
-            getPooledObject(pooledObject).use();
+            final PooledObject<T> po = getPooledObject(pooledObject);
+            if (po != null) {
+                po.use();
+            }
         }
     }
 
