@@ -30,7 +30,6 @@ import java.util.Arrays;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.HashSet;
-import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
@@ -100,12 +99,13 @@ public class TestPoolUtils {
         return createProxy(clazz, new MethodCallLogger(logger));
     }
 
-    private static List<String> invokeEveryMethod(final KeyedObjectPool<Object,Object> kop) throws Exception {
+    private static List<String> invokeEveryMethod(final KeyedObjectPool<Object, Object> kop) throws Exception {
         kop.addObject(null);
         kop.borrowObject(null);
         kop.clear();
         kop.clear(null);
         kop.close();
+        kop.getKeys();
         kop.getNumActive();
         kop.getNumActive(null);
         kop.getNumIdle();
@@ -114,8 +114,8 @@ public class TestPoolUtils {
         kop.returnObject(null, new Object());
         kop.toString();
 
-        return Arrays.asList("addObject", "borrowObject", "clear", "clear", "close", "getNumActive", "getNumActive",
-                "getNumIdle", "getNumIdle", "invalidateObject", "returnObject", "toString");
+        return Arrays.asList("addObject", "borrowObject", "clear", "clear", "close", "getKeys", "getNumActive", "getNumActive", "getNumIdle", "getNumIdle",
+                "invalidateObject", "returnObject", "toString");
     }
 
     private static <K, V> List<String> invokeEveryMethod(final KeyedPooledObjectFactory<K, V> kpof) throws Exception {
@@ -156,7 +156,7 @@ public class TestPoolUtils {
     }
 
     @Test
-    public void testCheckMinIdleKeyedObjectPool() throws Exception {
+    public void testCheckMinIdleKeyedObjectPool() throws InterruptedException {
         assertThrows(IllegalArgumentException.class, () -> PoolUtils.checkMinIdle(null, new Object(), 1, 1),
                 "PoolUtils.checkMinIdle(KeyedObjectPool,Object,int,long) must not allow null pool.");
         try (@SuppressWarnings("unchecked")
@@ -183,9 +183,7 @@ public class TestPoolUtils {
             assertEquals(2, kop.getNumIdle());
         }
         int makeObjectCount = 0;
-        final Iterator<String> iter = calledMethods.iterator();
-        while (iter.hasNext()) {
-            final String methodName = iter.next();
+        for (String methodName : calledMethods) {
             if ("makeObject".equals(methodName)) {
                 makeObjectCount++;
             }
@@ -226,7 +224,7 @@ public class TestPoolUtils {
     }
 
     @Test
-    public void testCheckMinIdleKeyedObjectPoolKeys() throws Exception {
+    public void testCheckMinIdleKeyedObjectPoolKeys() throws InterruptedException {
         // Because this isn't deterministic and you can get false failures, try more than once.
         AssertionFailedError afe = null;
         int triesLeft = 3;
@@ -278,7 +276,7 @@ public class TestPoolUtils {
     }
 
     @Test
-    public void testCheckMinIdleObjectPool() throws Exception {
+    public void testCheckMinIdleObjectPool() throws InterruptedException {
         assertThrows(IllegalArgumentException.class, () -> PoolUtils.checkMinIdle(null, 1, 1),
                 "PoolUtils.checkMinIdle(ObjectPool,,) must not allow null pool.");
         try (@SuppressWarnings("unchecked")
@@ -298,9 +296,7 @@ public class TestPoolUtils {
             assertEquals(2, op.getNumIdle());
         }
         int makeObjectCount = 0;
-        final Iterator<String> iter = calledMethods.iterator();
-        while (iter.hasNext()) {
-            final String methodName = iter.next();
+        for (String methodName : calledMethods) {
             if ("makeObject".equals(methodName)) {
                 makeObjectCount++;
             }
@@ -710,9 +706,10 @@ public class TestPoolUtils {
     }
 
     @Test
-    public void testSynchronizedPoolableFactoryKeyedPoolableObjectFactory() throws Exception {
-        assertThrows(IllegalArgumentException.class, () -> PoolUtils.synchronizedKeyedPooledFactory((KeyedPooledObjectFactory<Object, Object>) null),
-                "PoolUtils.synchronizedPoolableFactory(KeyedPoolableObjectFactory) must not allow a null factory.");
+    public void testSynchronizedPoolableFactoryKeyedPooledObjectFactory() throws Exception {
+        assertThrows(IllegalArgumentException.class,
+            () -> PoolUtils.synchronizedKeyedPooledFactory((KeyedPooledObjectFactory<Object, Object>) null),
+            "PoolUtils.synchronizedPoolableFactory(KeyedPooledObjectFactory) must not allow a null factory.");
 
         final List<String> calledMethods = new ArrayList<>();
         @SuppressWarnings("unchecked")
@@ -748,8 +745,8 @@ public class TestPoolUtils {
 
         final List<String> calledMethods = new ArrayList<>();
         try (@SuppressWarnings("unchecked")
-        final KeyedObjectPool<Object, Object> kop = createProxy(KeyedObjectPool.class, calledMethods);
-                final KeyedObjectPool<Object, Object> skop = PoolUtils.synchronizedPool(kop)) {
+            final KeyedObjectPool<Object, Object> kop = createProxy(KeyedObjectPool.class, calledMethods);
+            final KeyedObjectPool<Object, Object> skop = PoolUtils.synchronizedPool(kop)) {
             final List<String> expectedMethods = invokeEveryMethod(skop);
             assertEquals(expectedMethods, calledMethods);
         }
