@@ -179,12 +179,12 @@ public class TestAbandonedObjectPool {
         }
 
         @Override
-        public void destroyObject(final PooledObject<PooledTestObject> obj) throws Exception {
+        public void destroyObject(final PooledObject<PooledTestObject> obj) throws InterruptedException {
             destroyObject(obj, DestroyMode.NORMAL);
         }
 
         @Override
-        public void destroyObject(final PooledObject<PooledTestObject> obj, final DestroyMode destroyMode) throws Exception {
+        public void destroyObject(final PooledObject<PooledTestObject> obj, final DestroyMode destroyMode) throws InterruptedException {
             obj.getObject().setActive(false);
             // while destroying instances, yield control to other threads
             // helps simulate threading errors
@@ -218,7 +218,7 @@ public class TestAbandonedObjectPool {
 
     @SuppressWarnings("deprecation")
     @BeforeEach
-    public void setUp() throws Exception {
+    public void setUp() {
         abandonedConfig = new AbandonedConfig();
 
         // Uncomment the following line to enable logging:
@@ -264,7 +264,7 @@ public class TestAbandonedObjectPool {
             msg.append('\n');
             mbs.unregisterMBean(name);
         }
-        assertEquals( 0, registeredPoolCount,msg.toString());
+        assertEquals(0, registeredPoolCount, msg.toString());
     }
 
     /**
@@ -286,7 +286,7 @@ public class TestAbandonedObjectPool {
         final int n = 10;
         pool.setMaxTotal(n);
         pool.setBlockWhenExhausted(false);
-        pool.setTimeBetweenEvictionRuns(Duration.ofMillis(500));
+        pool.setDurationBetweenEvictionRuns(Duration.ofMillis(500));
         PooledTestObject obj = null;
         for (int i = 0; i < 5; i++) {
             obj = pool.borrowObject();
